/**
 * @file avltest.c
 * Demonstrate using an AVL tree database.
 *
 * Uses Daniel Nagy's GPL AVL library - see avl.c and avl.h for copyright.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "avl.h"

// ----------------------------------------------------------------

typedef struct avlnode
{
    struct avl avl;
    char *key;
    int  type;
    union {
        int   val;
        char *str;
        void *any;
    } t;
} Node_t;

int cmp(char *a, char *b)
{
    return strcmp(((Node_t*)b)->key,((Node_t*)a)->key);
}

char *newString(char *s)
{
    int len = 0;
    char *rs;

    while(s[++len] != 0)
        ;
    len++; // space for null terminator
    rs = (char*) malloc(len);
    //memcpy(rs, s, len);
    while(--len > -1)
        rs[len] = s[len];
    //printf("string: %d %s\n", len, rs);
    return rs;
}
enum { Int, String, Void };

void AvlNode(Node_t *node, char *k, int n, char *s, void *v)
{
    node->key = newString(k);
    node->type = s ? (String) : (v ? Void : Int);
    switch(node->type) {
        case Int:       node->t.val = n;
        break;
        case String:    node->t.str = s;
        break;
        case Void:      node->t.any = v;
        break;
    }
}

void listree(struct avl* a,int m)
{
    int n=m;
    if(a==0)
        return;
    if(a->right)
        listree(a->right,m+1);
    while(n--)
        printf("   ");
    printf("%s (%d)\n",((Node_t*)a)->key,a->balance);
    if(a->left)
        listree(a->left,m+1);
}

Node_t *find(struct avl *a, char *name, int incase)
{
    int comp = 0;
    char *key;
    if(a==0) return 0;
    key = ((Node_t*)a)->key;
    //printf("Enter find '%s' %d\n", key, strlen(key));
    comp = (incase > 0) ? strcmp(name, key) : strcasecmp(name, key);
    if(comp == 0) {
        //printf("Found '%s' %d\n", key, strlen(key));
        return (Node_t*)a;
    }
    else if(comp > 0)
        return find(a->left, name, incase);
    else if(comp < 0)
        return find(a->right, name, incase);

    return 0;
}


/* only for checking uniqueness */
int iterate(struct avl* a){
    if(a) {
        printf("%s\n",((Node_t*)a)->key);
        return 1;
    }
    return 0;
}

#define ITER 10
#define RMOD 1000

Node_t array[ITER];

int main(int argc, char **argv)
{
    int i;
    Node_t *node;

    avl_tree tree;
    tree.compar = cmp;
    tree.root = 0;

    AvlNode(&array[0], "Jimmy Hendrix", 0, 0, 0);
    AvlNode(&array[1], "Jimmy Page", 1, 0, 0);
    AvlNode(&array[2], "Ted Nugent", 2, 0, 0);
    AvlNode(&array[3], "Pete Carr", 3, 0, 0);
    AvlNode(&array[4], "JJ Bartlett", 4, "friend", 0);
    AvlNode(&array[5], "Yngwie Malmsteen", 5, "amazing", 0);
    AvlNode(&array[6], "Matt Denson", 6, "nephew", 0);
    AvlNode(&array[7], "Travis Wammack", 7, "scratchy", 0);
    AvlNode(&array[8], "Angus Young", 8, 0, 0);
    AvlNode(&array[9], "BB King", 9, 0, 0);

    printf("\nGuitar greats\n\n");

    for(i = 0; i < ITER; i++){
        printf("Add '%s' %d\n", array[i].key, array[i].t.val);
        //printf("++++++++++++++\n");
        avl_insert(&tree,(avl*)&array[i]);
        //listree(tree.root,0);
    }

    printf("\n你好, 凯兰\n");

    printf("\nShould Find: Yngwie malmsteen\n");
    if((node = find(tree.root, "Yngwie malmsteen", 0)))
        printf("Found %s type %d\n", node->key, node->type);

    printf("\nShould Find: Pete Carr\n");
    if((node = find(tree.root, "Pete Carr", 0)))
        printf("Found %s type %d\n", node->key, node->type);

    printf("\nDon't Find: travis wammack - case sensitive\n");
    if((node = find(tree.root, "travis wammack", 1)))
        printf("Found %s type %d\n", node->key, node->type);

    for(i = 0; i < ITER; i++){
        //printf("-------------\n");
        avl_remove(&tree,(struct avl*)&array[i]);
        //listree(tree.root,0);
    }

    printf("Don't Find: Pete Carr - all deleted\n");
    if((node = find(tree.root, "Pete Carr", 0)))
        printf("Found %s type %d\n", node->key, node->type);

    printf("\n");
  
    return 0;
}
